# constants
R = 8.3145  # J/mol K; ideal gas constant
F = 96485 # C/mol e-; Faraday's constant

# densities
rho_h2 = 0.08988  # kg/m^3 or g/L; hydrogen gas at STP
rho_o2 = 1.4291  # kg/m^3 or g/L; oxygen gas at STP
rho_co2 = 1.977  # kg/m^3 or g/L; carbon dioxide at STP
rho_h2o = 1000  # kg/m^3 or g/L; liquid water
rho_ch3oh = 786.3  # kg/m^3; liquid methanol

# molecular weights
mol_weight_h2o = 18.0146e-3  # kg/mol; H2O
mol_weight_h2 = 2.01588e-3  # kg/mol; H2
mol_weight_o2 = 31.998e-3  # kg/mol; O2
mol_weight_co = 28.01e-3  # kg/mol; CO
mol_weight_co2 = 44.0095e-3  # kg/mol; CO2
mol_weight_ch3oh = 32.042e-3  # kg/mol; CH3OH
mol_weight_nacl = 58.44e-3  # kg/mol; NaCl
